// Scintilla source code edit control
// scintilla-ext.cxx - This file provides access to the widget services required by
//                  code outside(aka "EXTernal") of the widget implementation
// Copyright (c) 2013 Mentor Graphics Corporation
// The license.terms file describes the conditions under which this software may be distributed.

#include "scintilla-ext.h"
#include "scintilla-cmd.h"

using namespace TkSci;

namespace TkSciExt {

//==================================================
// Return the current width of the visible margins
//==================================================
int GetMarginWidth(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;
   return sciw->marginMgr->Width();
}

//==================================================
// Store the main pixmap that we get from the
// platform code
//==================================================
void SetPixmap(ClientData d, Pixmap p) {

   SciWidget *sciw = (SciWidget *)d;
   sciw->mainPixmap = p;
}

//==================================================
// Add the specified Y delta to the current value
// being maintained in the widget
//==================================================
void SetYScrollDelta(ClientData d, int delta) {

   SciWidget *sciw = (SciWidget *)d;
   sciw->yscrollDelta += delta;
}

//==================================================
// Schedule an update to redraw following a
// scroll change initiated by the user
//==================================================
void UpdateForScroll(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;

   if ( !(sciw->flags & REDRAW_PENDING) ) {
//printf("ScrollText -- QUEUING DISPLAY\n");
	 Tk_CancelIdleCall(SciWidgetDisplay, (ClientData) d);
     Tk_DoWhenIdle(SciWidgetDisplay, d);

   } else {
//printf("ScrollText -- SKIPPING REDRAW...\n");
   }
   sciw->flags |= REDRAW_YSCROLL;
}

//==================================================
// Callback from Scintilla that lines were added.
// Note, if the "added" value is <0 it means that
// lines were removed instead of added.
//==================================================
void MSG_LinesAdded(ClientData d, int line, int added) {

   SciWidget *sciw = (SciWidget *)d;

   sciw->added_data->line  = LINE2TK(line);
   sciw->added_data->added = added;

   MessageFromScintilla(sciw, SCIW_LINESADDED_UPDATE);
}

//==================================================
// Callback from Scintilla to change mouse cursor
//==================================================
void MSG_SetCursor(ClientData d, int cursor_id) {

   SciWidget *sciw = (SciWidget *)d;

   //TODO: this needs a better way to indicate what
   //cursor is being request
   MessageFromScintilla(sciw, static_cast<int>(cursor_id));
}

//==================================================
// Callback from Scintilla that we use to notify
// client so it can respond to a change
// to the current document contents.
//==================================================
void MSG_Modified(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;

   MessageFromScintilla(sciw, SCIW_MODIFIED);
}

//==================================================
// Callback from Scintilla that we use to notify
// client so it can respond to a possible change
// to the current document position.
//==================================================
void MSG_MoveUpdate(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;

   MessageFromScintilla(sciw, SCIW_MOVE_UPDATE);
}

//==================================================
// Sync the widget's scroll position data based on
// a programmatic change from Scintilla
//==================================================
void MSG_SyncXScroll(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;
   MessageFromScintilla(sciw, SCIW_SYNC_XSCROLL);
}
void MSG_SyncYScroll(ClientData d) {

   SciWidget *sciw = (SciWidget *)d;
   MessageFromScintilla(sciw, SCIW_SYNC_YSCROLL);
}

}//--- end of TkSciExt namespace
