# location of Tk text widget source code:
# /u/prod_dev/tcltk/src/8.5.a5/tk/generic/tkText.c

set DO_TEXT  0 ;# create a std Tk Text widget?
set DO_SBAR  1 ;# create scrollbar?
set DO_FILE  1 ;# load a file?
set DO_FOCUS 1 ;# put focus into widget?
set DO_LANG  1 ;# use syntax highlighting?
set DO_BINDS 1 ;# do we load the default bindings?
set DO_LNUMS 1 ;# show line numbers?
set DO_IMGS  1 ;# show image-based markers?
set DO_FOLD  1 ;# show fold margin

# load the shared library
set SUFFIX ""
if {[info exists ::env(WINDIR)] && [info exists ::tcl_platform(debug)]} {
    set SUFFIX "g"
}

# load the shared library, assumes test being run in build directory
set auto_path [linsert auto_path 0 [file normalize [glob ./usr/lib/ScintillaTk*]]]
package require ScintillaTk

##################
# TEST
##################
set curr_loc ""
proc moveupdate {args} {
   set ix [$::sci index insert]
   ::scan $ix {%d.%d} row col
   set ::curr_loc [format "RC: %d, %d" $row $col]
}
proc addedupdate {args} {
#   puts "ADDED UPDATE: args=$args"
   eval $::sci margin filln LINENUMBERS $args
}
proc clicked {args} {
   ::lassign $args margin line x y btn shift ctrl alt
   puts "MARGIN CLICK: margin=$margin line=$line X,Y=$x,$y btn=$btn shift=$shift ctrl=$ctrl alt=$alt"
}
set scif .cf
destroy $scif .t

# create the widget
frame $scif
set sci $scif.c
scintilla $sci
set rpad [frame $scif.rpad -width 4] ;# << "internal" pad on RHS of text area
$scif config -bg [$sci cget -bg]
$rpad config -bg [$sci cget -bg]
pack $rpad -side right -fill y
pack $sci -side left -fill both -expand 1

# configure callbacks
$sci config -moveupdatec moveupdate -linesa addedupdate

$sci configure -marginbackground red

# configure appearance
$sci tag config exec -margin -fg red


# configure the margins we want
$sci margin config 0 -type text      -alias cov1
$sci margin config 1 -type symbol    -alias cov2          -clickc clicked
$sci margin config 2 -type lnums_alt -alias LINENUMBERS   -clickc clicked
$sci margin config 3 -type symbol    -alias marks -wid 18 -clickc clicked
$sci margin config 4 -type fold

if {$DO_SBAR} {
   ttk::scrollbar .vsb -command "$sci yview"
   ttk::scrollbar .hsb -command "$sci xview" -orient horizontal
   $sci config -yscrollcommand ".vsb set"
   $sci config -xscrollcommand ".hsb set"

   frame .f
   button .f.b1 -text "-" -command "$sci zoomout" -repeatdelay 500 -repeatint 100
   button .f.b2 -text "+" -command "$sci zoomin"  -repeatdelay 500 -repeatint 100
   label .f.loc -textvariable ::curr_loc -font {-family courier} -relief sunken
   label .f.l -text "Zoom:"
   pack .f.loc -side right
   pack .f.l .f.b1 .f.b2 -side left

   grid $scif  -row 0 -column 0 -sticky nswe
   grid .vsb -row 0 -column 1 -sticky ns
   grid .hsb -row 1 -column 0 -sticky we
   grid .f   -row 2 -column 0 -sticky we
   grid rowconfig    . 0 -weight 1
   grid columnconfig . 0 -weight 1
} else {
   pack $scif -fill both -expand 1
}

set lang "none"

if {[llength $argv] > 0} {
	set f [lindex $argv 0]
} else {
set EX ../../src/tk
set f $EX/PlatTK.cxx
}

if {$DO_LANG} {
switch [::file extension $f] {
   .tcl                 {$sci config -lang tcl}
   .sv - .svh - .v      {$sci config -lang verilog}
   .c - .cxx - .cpp     {$sci config -lang cpp}
   .vhd - .vhdl         {$sci config -lang vhdl}
   .htm - .html - .rmdb {$sci config -lang html}
}
}
if {$DO_TEXT} {
   toplevel .t
   wm title .t "Tk Text"
   ttk::scrollbar .t.vsb -command ".t.t yview"
   ttk::scrollbar .t.hsb -command ".t.t xview" -orient horizontal
   text .t.t -wrap none -yscrollc ".t.vsb set" -xscrollc ".t.hsb set"
   grid .t.t   -row 0 -column 0 -sticky nswe
   grid .t.vsb -row 0 -column 1 -sticky ns
   grid .t.hsb -row 1 -column 0 -sticky we
   grid rowconfig    .t 0 -weight 1
   grid columnconfig .t 0 -weight 1
#    pack .t.vsb -side right -fill y    -expand 0
#    pack .t.t   -side left  -fill both -expand 1
}
if {$DO_FILE} {
   set fd [open $f r]
   puts [format "FILE LOAD TIME1: %s" [lrange [time [list $sci insert end [read $fd]]] 0 1]]
   if {$DO_TEXT} {
      seek $fd 0
      puts [format "FILE LOAD TIME2: %s" [lrange [time [list .t.t insert end [read $fd]]] 0 1]]
   }
   close $fd
   if {$DO_LNUMS} {
      $sci margin show LINENUMBERS ;# do it here so we'll know how many lines are in the file
   }
   wm title . "Scintilla TK ($f)"
} else {
   wm title . "Scintilla TK"
}
if {$DO_FOCUS} {
   focus $sci
}
. config -bg red ;# <<< this will show if we're missing something in the paint code

set xpm_pointer {/* XPM */
static char * InlineData[] = {
"16 16 14 1",
"  s None c None",
". c #0080ff",
"# c #006ce0",
"a c #0000c0",
"b c #3b9bff",
"c c #3b70ea",
"d c #0076ef",
"e c #58a8ff",
"f c #002cc0",
"g c #168aff",
"h c #162ad0",
"i c #0042c0",
"j c #0058c0",
"k c #2c54e0",
"                ",
"       j        ",
"       #b       ",
"       #.b      ",
"       #..g     ",
"j######d...ge   ",
"j............b  ",
"j#############j ",
"jaaaaaaaaaaaahj ",
"jaaaaaaaaaaak   ",
"ajjjjjjfaaac    ",
"       fahc     ",
"       fh       ",
"       i        ",
"                ",
"                "};}
set xpm_rightarrow {/* XPM */
static char * InlineData[] = {
"16 16 76 2",
"   s None c None",
".. c #0543df",
"#. c #083bcc",
"a. c #0850e2",
"b. c #0659f0",
"c. c #065af4",
"d. c #073dc7",
"e. c #0551eb",
"f. c #3d9cfd",
"g. c #0127b7",
"h. c #0360fb",
"i. c #065cf8",
"j. c #7bbefd",
"k. c #1789fd",
"l. c #0127b3",
"m. c #0668fc",
"n. c #025df4",
"o. c #0752eb",
"p. c #3c96fc",
"q. c #0f68fb",
"r. c #0649e2",
"s. c #0940cb",
"t. c #247ffc",
"u. c #0637c2",
"v. c #3090fc",
"w. c #0638c6",
"x. c #063cd6",
"y. c #064fe4",
"z. c #55acfd",
"A. c #093bc9",
"B. c #91c4fd",
"C. c #0350ed",
"D. c #71b9fd",
"E. c #419efd",
"F. c #083fcc",
"G. c #0841d4",
"H. c #022fc2",
"I. c #0460f8",
"J. c #4fa5fd",
"K. c #69b2fd",
"L. c #3291fc",
"M. c #084fe0",
"N. c #57adfd",
"O. c #0548df",
"P. c #0550e9",
"Q. c #022abc",
"R. c #0e79fe",
"S. c #a1ccfd",
"T. c #0740d1",
"U. c #0743dd",
"V. c #0748db",
"W. c #0749df",
"X. c #0128b9",
"Y. c #073cd3",
"Z. c #1074fc",
"0. c #0750e5",
"1. c #33a5fd",
"2. c #0335c5",
"3. c #074deb",
"4. c #0457f0",
"5. c #032fbf",
"6. c #025ffa",
"7. c #093ac7",
"8. c #b6d5fc",
"9. c #3c92fc",
".# c #064ae4",
"## c #1e7cfc",
"a# c #0535c1",
"b# c #4fa9fd",
"c# c #2796fd",
"d# c #0650e6",
"e# c #0351ef",
"f# c #43a3fd",
"g# c #0842d6",
"h# c #086ffc",
"i# c #4db5fd",
"                                ",
"                  a.5.          ",
"                  y.9.5.        ",
"                  d#p.J.a#      ",
"    W.M.0.o.b.i.q.t.f.b#K.u.    ",
"    g#U..#o.c.q.##v.E.z.D.B.w.  ",
"    g#U..#3.c.q.##L.f#N.j.S.8.d.",
"    7.7.7.7.#.x.O.e#6.h#k.1.i#2.",
"    7.7.7.7.A.Y...C.n.m.R.c#H.  ",
"    G.G.G.G.G.V.r.P.4.h.Z.Q.    ",
"                  T.P.I.X.      ",
"                  F.e.g.        ",
"                  s.l.          ",
"                                ",
"                                ",
"                                "};}
set xpm_rainbow {/* XPM */
static char * InlineData[] = {
"16 16 199 2",
"   s None c None",
".. c #faceac",
"#. c #f8ed8d",
"a. c #1d6494",
"b. c #fc0e2b",
"c. c #1a8693",
"d. c #f53780",
"e. c #705fbd",
"f. c #f9dd9d",
"g. c #f24dac",
"h. c #6c43ff",
"i. c #ed6ae7",
"j. c #f054bb",
"k. c #9955ff",
"l. c #4457a7",
"m. c #695eba",
"n. c #ebf388",
"o. c #245297",
"p. c #f348a1",
"q. c #12e190",
"r. c #ff81fa",
"s. c #fda0db",
"t. c #f8e496",
"u. c #f479ff",
"v. c #1c7094",
"w. c #f056bf",
"x. c #35fd8d",
"y. c #fbbdbd",
"z. c #ca6eea",
"A. c #f24ba9",
"B. c #eb73fa",
"C. c #4458a7",
"D. c #fca9d3",
"E. c #fcafcc",
"F. c #f63479",
"G. c #facead",
"H. c #fca7d4",
"I. c #1b7b94",
"J. c #ff82fa",
"K. c #37fd8d",
"L. c #bcf589",
"M. c #189f92",
"N. c #ebf289",
"O. c #c46de7",
"P. c #1e6694",
"Q. c #fa1d49",
"R. c #f3459a",
"S. c #b1f68a",
"T. c #baf689",
"U. c #fbb0cc",
"V. c #ed6eef",
"W. c #199592",
"X. c #de71f4",
"Y. c #705fbe",
"Z. c #edf289",
"0. c #5039ff",
"1. c #f6367d",
"2. c #bb62ff",
"3. c #f9db9f",
"4. c #f6f288",
"5. c #ef5fd2",
"6. c #ca68ff",
"7. c #f43d8c",
"8. c #3efc8d",
"9. c #f3479e",
".# c #f348a2",
"## c #ec70f3",
"a# c #f8265c",
"b# c #ee65dd",
"c# c #d16fee",
"d# c #f05ac7",
"e# c #73fa8c",
"f# c #7ff98b",
"g# c #f53985",
"h# c #17ab92",
"i# c #1b7f93",
"j# c #fbbdbe",
"k# c #f92255",
"l# c #ed68e5",
"m# c #bc63ff",
"n# c #fc163b",
"o# c #1a8a93",
"p# c #f151b5",
"q# c #8b64cb",
"r# c #f3469f",
"s# c #17ac92",
"t# c #fcabcf",
"u# c #8c64cb",
"v# c #fad2a8",
"w# c #c9f489",
"x# c #32549f",
"y# c #4758a8",
"z# c #f058c4",
"A# c #50fb8c",
"B# c #11fe8e",
"C# c #f7ef8a",
"D# c #12dd8f",
"E# c #7ff88c",
"F# c #10f28f",
"G# c #eb75ff",
"H# c #fd95e6",
"I# c #85f98b",
"J# c #9f58ff",
"K# c #f152b5",
"L# c #d870f2",
"M# c #f9d9a1",
"N# c #fc1334",
"O# c #f8265d",
"P# c #f53c89",
"Q# c #af5eff",
"R# c #1b8193",
"S# c #f7306f",
"T# c #f059c4",
"U# c #ec6be9",
"V# c #a158ff",
"W# c #f24eae",
"X# c #ee60d3",
"Y# c #fb1a43",
"Z# c #3b56a2",
"0# c #ed6ced",
"1# c #9352ff",
"2# c #18a392",
"3# c #ef5fcf",
"4# c #fd9cdf",
"5# c #fbc3b8",
"6# c #c3f58a",
"7# c #59fa8c",
"8# c #1e5694",
"9# c #fa2152",
".a c #b36adf",
"#a c #f9daa1",
"aa c #1c6b94",
"ba c #f82861",
"ca c #f73377",
"da c #cdf589",
"ea c #f53982",
"fa c #fd7cff",
"ga c #ee63db",
"ha c #ee62d7",
"ia c #e070ff",
"ja c #19fe8e",
"ka c #50fb8d",
"la c #fac2b9",
"ma c #1a8d93",
"na c #f8ef8b",
"oa c #11ea8f",
"pa c #1a8393",
"qa c #fd95e7",
"ra c #fda3d8",
"sa c #17b191",
"ta c #eb70f5",
"ua c #f82a65",
"va c #a0f68a",
"wa c #f347a0",
"xa c #f9d9a2",
"ya c #ee77ff",
"za c #95f78b",
"Aa c #d4f489",
"Ba c #10f58f",
"Ca c #f73174",
"Da c #f73070",
"Ea c #f059c5",
"Fa c #96f78b",
"Ga c #a167d6",
"Ha c #f44191",
"Ia c #f054ba",
"Ja c #ed6ae6",
"Ka c #fac9b2",
"La c #f6377f",
"Ma c #3c30ff",
"Na c #fac1ba",
"Oa c #3455a0",
"Pa c #a0f78a",
"Qa c #8262c6",
"Ra c #3d30ff",
"Sa c #ec71f5",
"Ta c #ec72f9",
"Ua c #18a891",
"Va c #3555a0",
"Wa c #f8275e",
"Xa c #f14faf",
"Ya c #fe091f",
"Za c #fbbfbb",
"0a c #a2f78a",
"1a c #a969d9",
"2a c #ee62d8",
"3a c #1c7093",
"4a c #fb1c48",
"5a c #f72e6d",
"6a c #5c5cb3",
"7a c #3c31ff",
"8a c #d36cff",
"9a c #a7f68a",
".b c #22fe8e",
"#b c #fac8b3",
"ab c #2efd8d",
"bb c #fca1da",
"cb c #3d31ff",
"db c #ef5dcd",
"    cbcb7acbcb1#J#cbRacbcbcb    ",
"  7a7acb6.bb#bM#3.G.U.ya7aRaMa  ",
"7acbcbqaf.vaK.oaq..bf##.E.V#cbcb",
"7a7as.n..bpax#m.e.y#3aD#S.y.1#7a",
"cbJ.naBa8#.a0#b#gai.L#l.s#T.H.cb",
"0.Kakaa.c#haK#p.r#W#dbSaZ#F#t.8a",
"G#4.h#Qab#g.eaS#5a1.wa3#O.i#0a4#",
"H#9aI.X.z#7.ua4aY#a#d.p#V.o.A#j#",
"raI#P.SaIaLak#b.YaQ.CaA.l#l.ab#b",
"4#zav.Taw.g#O#n#N#9#F.XaU#Va8.5#",
"r.AaW.1aX#9.cabaWaDaHaT#B.aaE#t#",
"Q#M#jaOa##d#.#7.P#R.j.JaY.saZ.fa",
"7as.L.2#6ata2ad#Ea5.0#q#R#e#Nacb",
"Rak.laFaUao.u#z.c#GaC.ma7##aiacb",
"  cb2.Zadax.h#o#c.M.B#Pav#u.cb  ",
"    cbh.H#..C#w#6#N.xaD.m#cb    "};}

puts "LOADED widget.tcl"

if {$DO_IMGS} {
    set img1 [image create photo -file $EX/test/pointer.gif]
	$sci marker configure pointer -image $img1

	set img2 [image create photo -file $EX/test/rightarrow.gif]
	$sci marker configure rightarrow -image $img2

	set img3 [image create photo -file $EX/test/rainbow.gif]
    $sci marker configure rainbow -image $img3

    $sci margin marker 3 add pointer 16
    $sci margin marker 3 add rainbow 17
    $sci margin marker 3 add rightarrow 19
}

if {$DO_FOLD} {
	$sci margin show fold
}
